// PrintJob_Gdi.cs - Creates a print job by creating a DC
// and calling GDI drawing calls.
//
// Code from _Programming the .NET Compact Framework with C#_
// and _Programming the .NET Compact Framework with VB_
// (c) Copyright 2002-2004 Paul Yao and David Durant. 
// All rights reserved.

using System;
using System.IO;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using IDAutomationPPC.Win32;
using IDAutomationPPC.Drawing;

namespace PrintGdi
{
	public class PrintJob_Gdi
	{
		/// <summary>
		/// pointer to bitmap info structure passed to drawing function
		/// </summary>
		private static IntPtr bitmapInfoPtr;

		/// <summary>
		/// Default width of image being printed
		/// </summary>
		const int imageWidth = 100;
		
		/// <summary>
		/// Default height of image being printed
		/// </summary>
		const int imageHeight = 100;

		/// <summary>
		/// handle to pixels that we allocate for image
		/// </summary>
		private static GCHandle pixelsHandle;

		/// <summary>
		/// Public function that sends text to the printer
		/// </summary>
		/// <param name="textIn"></param>
		/// <param name="hdc"></param>
		public static void PrintText(TextBox textIn, IntPtr hdc)
		{
			// Split input data into separate lines of text.
			char [] achNewLine = new char[] { '\n'};
			String [] astrSplit;
			astrSplit = textIn.Text.Split(achNewLine);

			// Calculate longest string in the document
			int cchMax = 0;
			int cstr = astrSplit.Length;
			for (int i = 0; i < cstr; i++)
			{
			if (astrSplit[i].Length > cchMax)
				cchMax = astrSplit[i].Length;
			}

			// Allocate conversion buffers.
			byte[] byteData = new Byte[cchMax];
			char[] chData = new Char[cchMax];
			System.Text.Encoder d;
			d = System.Text.Encoding.UTF8.GetEncoder();

			// Get device resolution
			int cxyInch =
			GdiGraphics.GetDeviceCaps(hdc, CAPS.LOGPIXELSY);
			// In draft mode, the PCL driver returns wrong value.
			if (cxyInch == 0) 
			{ 
			cxyInch = 150; 
			}

			// Calculate page size.
			int cxPhysPage =
			GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALWIDTH);
			int cyPhysPage = 
			GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALHEIGHT);
			int cxOff = 
			GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALOFFSETX);
			int cyOff = 
			GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALOFFSETY);

			int cxHorzRes = GdiGraphics.GetDeviceCaps(hdc, CAPS.HORZRES);
			int cyVertRes = GdiGraphics.GetDeviceCaps(hdc, CAPS.VERTRES);

			// Calculate line height.
			TEXTMETRIC tm = new TEXTMETRIC();
			GdiFont.GetTextMetrics(hdc, ref tm);
			int cyLineHeight = tm.tmHeight + tm.tmExternalLeading;

			// Init text drawing coordinates;
			int xText = cxyInch - cxOff;
			int yText = cxyInch - cyOff;

			// Calculate page boundaries
			int yFirst = yText;
			int yLast  = cyPhysPage - cxyInch;

			try
			{
			// Notify GDI of document and page start.
			DOCINFO di = new DOCINFO();
			di.cbSize = Marshal.SizeOf(di);
			Printing.StartDoc(hdc, ref di);
			Printing.StartPage(hdc);

			// Set iEnd -- trim extra carriage-return from text
			int iEnd = 0;
			int cchString = astrSplit[0].Length;
			char ch = astrSplit[0][cchString-1];
			if (ch == '\r') iEnd = -1;

			// Loop through list of strings.
			for (int i = 0; i < cstr; i++)
			{
				cchString = astrSplit[i].Length;
				if (cchString > 0)
				{
					// Draw line of text.
					GdiGraphics.ExtTextOut(hdc, xText, yText, 0,
						IntPtr.Zero, astrSplit[i], cchString + iEnd,
						IntPtr.Zero);
				}

				// Advance to next line.
				yText += cyLineHeight;

				// Skip to next page (if not at document end)
				if (yText >= yLast && (i+1) < cstr)
				{
					Printing.EndPage(hdc);
					Printing.StartPage(hdc);
					yText = yFirst;
				}
			}
			}
			finally
			{
			// End of page & end of document.
			Printing.EndPage(hdc);
			Printing.EndDoc(hdc);
			}

		} // PrintText()

	  
	   /// <summary>
	   /// Draws a bitmap on a graphics object
	   /// </summary>
	   /// <param name="pixels">an array of pixels representing the image</param>
	   /// <param name="hdc">The Device Context of the printer</param>
	   //--------------------------------------------------------
	   public static void PrintGraphics(byte[] pixels, IntPtr hdc)
	   {
		   // Allocate conversion buffers.
		   
		   System.Text.Encoder d;
		   d = System.Text.Encoding.UTF8.GetEncoder();

		   // Get device resolution
		   int cxyInch = GdiGraphics.GetDeviceCaps(hdc, CAPS.LOGPIXELSY);
		   
		   // In draft mode, the PCL driver returns wrong value.
		   if (cxyInch == 0) 
		   { 
			   cxyInch = 150; 
		   }

		   // Calculate page size.
		   int cxPhysPage =
			   GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALWIDTH);
		   int cyPhysPage = 
			   GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALHEIGHT);
		   int cxOff = 
			   GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALOFFSETX);
		   int cyOff = 
			   GdiGraphics.GetDeviceCaps(hdc, CAPS.PHYSICALOFFSETY);

		   int cxHorzRes = GdiGraphics.GetDeviceCaps(hdc, CAPS.HORZRES);
		   int cyVertRes = GdiGraphics.GetDeviceCaps(hdc, CAPS.VERTRES);

		   // Init text drawing coordinates;
		   int xText = cxyInch - cxOff;
		   int yText = cxyInch - cyOff;

		   // Calculate page boundaries
		   int yFirst = yText;
		   int yLast  = cyPhysPage - cxyInch;

		   try
		   {
			   // Notify GDI of document and page start.
			   DOCINFO di = new DOCINFO();
			   di.cbSize = Marshal.SizeOf(di);
			   Printing.StartDoc(hdc, ref di);
			   Printing.StartPage(hdc);
	   
			   //Create a gropu of pixels based on the image that is passed in
			   //byte[] pixels;
			   IntPtr pixelsPtr;			   

//			   pixels = new byte[imageWidth * imageHeight];   // sample image
//
//			   for(int i = 0; i < pixels.Length; i++)    // fill it by some way
//			   {
//				   pixels[i] = (byte)i;
//			   }

			   // make unmanaged pointer to pixels (you must do this for every image)
			   pixelsHandle = GCHandle.Alloc(pixels, GCHandleType.Pinned);
			   pixelsPtr = pixelsHandle.AddrOfPinnedObject();
			   CreateBitmapInfo();

			   //Draw the bitmap
			   GdiGraphics.StretchDIBits(hdc, 0, 0, 1000, 1000,0, 0, 1000, 1000, 
				   pixels, bitmapInfoPtr, GdiGraphics.DIB_RGB_COLORS, GdiGraphics.SRCCOPY);
				   
		   }
		   finally
		   {
			   // End of page & end of document.
			   Printing.EndPage(hdc);
			   Printing.EndDoc(hdc);
		   }
	   }// PrintGraphics()

		//Create wrappers for the memory API's similar to
		//Marshal.AllocHGlobal and Marshal.FreeHGlobal
		internal static IntPtr AllocHGlobal(IntPtr cb)
		{
			IntPtr hMemory = new IntPtr();
			hMemory = GdiGraphics.LocalAlloc(GdiGraphics.LPTR, (uint)cb);
			return hMemory;
		}

		//got to get rid of the memory that we allocated
		public static void Dispose()
		{
			// Release handle to pixels. You must do this for every frame
			pixelsHandle.Free();

			FreeHGlobal(bitmapInfoPtr);
		}

		//wrapper class because CF does not support native FreeHGlobal
		internal static void FreeHGlobal(IntPtr hMemory)
		{
			if (hMemory != IntPtr.Zero)
				GdiGraphics.LocalFree(hMemory);
		}

		// allocate BITMAPINFO and color palette in unmanaged memory
		private static void CreateBitmapInfo()
		{
			IDAutomationPPC.Drawing.BITMAPINFO info = new IDAutomationPPC.Drawing.BITMAPINFO();

			info.biSize = Marshal.SizeOf(typeof(IDAutomationPPC.Drawing.BITMAPINFO)) - Marshal.SizeOf(typeof(Int32));  // sizeof BITMAPINFOHEADER
			info.biWidth = imageWidth;
			info.biHeight = imageHeight;
			info.biPlanes = 1;
			info.biBitCount = 8;
			info.biCompression = 0;     // BI_RGB
			info.biSizeImage = imageWidth * imageHeight;
			info.biXPelsPerMeter = 0;
			info.biYPelsPerMeter = 0;
			info.biClrUsed = 0;
			info.biClrImportant = 0;
			info.bmiColors = 0;         // to prevent warning

			// allocate memory
			bitmapInfoPtr = AllocHGlobal((IntPtr)(Marshal.SizeOf(typeof(BITMAPINFO)) +
				Marshal.SizeOf(typeof(RGBQUAD)) * 256));

			// copy BITMAPINFO
			Marshal.StructureToPtr(info, bitmapInfoPtr, false);

			int offset = Marshal.SizeOf(typeof(IDAutomationPPC.Drawing.BITMAPINFO)) - Marshal.SizeOf(typeof(Int32));

			IDAutomationPPC.Drawing.RGBQUAD color = new RGBQUAD();

//			// copy color palette
//			for ( int i = 0; i < 256; i++ )
//			{
//				color.rgbRed = (byte)i;
//				color.rgbGreen = (byte)i;
//				color.rgbBlue = (byte)i;
//				color.rgbReserved = 0;
//
//				IntPtr p = new IntPtr(bitmapInfoPtr.ToInt32() + offset);
//
//				Marshal.StructureToPtr(color, p, false);
//
//				offset += Marshal.SizeOf(typeof(IDAutomationPPC.Drawing.RGBQUAD));
//			}
		}//End CreateBitmapInfo()
	} // class
} // namespace
